
##### Houskeeping #####
rm(list=ls())

options(java.parameters = "-Xmx2048m")

library('dplyr')
library('haven')
library('gender')
library('stringdist')
library('assertthat')
library('readr')
library('rstudioapi')

localeToCharset(locale = Sys.getlocale("LC_CTYPE"))

# Set path
setwd('')

print(getwd())

# Timer
startTime<-proc.time()



##### Get lists of unique names #####

# Load EconLit
load('temp/UniqueNames.RData')



##### Use R package 'gender'#####
### Use "gender" package to predict probabilities
### "gender" package by maintained by Lincoln Mullen (Assistant Professor at George Mason University)
### Other authors are Cameron Blevins and Benjamin Schmidt (assistant professors at Northeastern University)
### Source for the method I use in the package is the US Social Security Administration 

Gender_SSA = gender(UniqueFirstNames, method='ssa')[c('name', 'proportion_male')]
Gender_SSA = dplyr::rename(Gender_SSA, pMale_SSA=proportion_male)



##### Match with RePEc data on top F economists #####

# Load RePEc data
load('input/repec_TopFemale_Clean.RData')
repec$female_name = as.character(repec$female_name)


Gender_Repec = as.data.frame(UniqueFullNames)

# Use amatch to get a small-tolerance match
Gender_Repec$MatchIndex = sapply(UniqueFullNames, function(x) {
  amatch(x, repec$female_name, nomatch=NA, maxDist=1) })

# Fill in with matched names for reference
Gender_Repec$Repec_MatchName = repec$female_name[Gender_Repec$MatchIndex]
Gender_Repec$Repec_F = !is.na(Gender_Repec$MatchIndex)



##### Match with German data on names #####

load('input/German_Gender_Cleaned.RData')



##### Match with EEA Women in Econ list #####

load('input/EEA_Female.RData')

Gender_EEA = as.data.frame(UniqueFullNames)

# Use amatch to get a small-tolerance match
Gender_EEA$MatchIndex = sapply(UniqueFullNames, function(x) {
  amatch(x, EEA_WomenEcon$Name, nomatch=NA, maxDist=1) })

# Use merge to match exact names for Override list
Gender_EEA$MatchIndexOverride = sapply(UniqueFullNames, function(x) {
  match(x, EEA_WomenEcon_Override$Name, nomatch=NA) })

# Fill in with matched names for reference
Gender_EEA$EEA_MatchName = EEA_WomenEcon$Name[Gender_EEA$MatchIndex]
Gender_EEA$EEA_MatchNameOverride = EEA_WomenEcon_Override$Name[Gender_EEA$MatchIndexOverride]

Gender_EEA$EEA_F = !is.na(Gender_EEA$MatchIndex) | !is.na(Gender_EEA$MatchIndexOverride)


##### Open URAP Audits #####

# Original
load('input/URAP_AuditList.RData')
Gender_URAP = Gender_URAP[order(Gender_URAP$Gender),]
Gender_URAP = dplyr::rename(Gender_URAP, Gender_URAP=Gender)

# REStud
load('input/URAP_AuditList_REStud.RData')
Gender_URAP_REStud = Gender_URAP_REStud[order(Gender_URAP_REStud$Gender),]
Gender_URAP_REStud = dplyr::rename(Gender_URAP_REStud, Gender_URAP=Gender)

# QJE
load('input/URAP_AuditList_QJE.RData')
Gender_URAP_QJE = Gender_URAP_QJE[order(Gender_URAP_QJE$Gender),]
Gender_URAP_QJE = dplyr::rename(Gender_URAP_QJE, Gender_URAP=Gender)

# QJE
load('input/URAP_AuditList_REStat.RData')
Gender_URAP_REStat = Gender_URAP_REStat[order(Gender_URAP_REStat$Gender),]
Gender_URAP_REStat = dplyr::rename(Gender_URAP_REStat, Gender_URAP=Gender)


# Append URAP audits
Gender_URAP <- rbind(Gender_URAP, Gender_URAP_REStud, Gender_URAP_QJE)
Gender_URAP <- Gender_URAP[!duplicated(Gender_URAP$Name),]
Gender_URAP$Gender_URAP[Gender_URAP$Gender_URAP!="F" & Gender_URAP$Gender_URAP!="M"] <- 'U'



##### Open Chinese names #####

load('input/Chinese_Names.RData')
ChineseSurnames$CommonCH_Surname <- 1
ChineseGivenNames$CommonCH_Given <- 1


##### Create dataset to match later #####

# Create list of names to be matched
Gender_Names = UniqueNames

# SSA
Gender_Names = merge(Gender_Names, Gender_SSA, by.x = 'FirstName', by.y='name', all.x=TRUE, all.y=FALSE)

# German names
Gender_Names = merge(Gender_Names, Gender_German_Wide[c('Name', 'pMale_German', 'pFemale_German')], by.x='FirstName', by.y='Name', all.x=TRUE, all.y=FALSE)

# EEA Women in Econ names
Gender_Names = merge(Gender_Names, Gender_EEA[c('UniqueFullNames', 'EEA_F')], by.x='Name', by.y='UniqueFullNames', all.x=TRUE, all.y=FALSE)
Gender_Names$EEA_F = Gender_Names$EEA_F*1

# RePEc
Gender_Names = merge(Gender_Names, Gender_Repec[c('UniqueFullNames', 'Repec_F')], by.x='Name', by.y='UniqueFullNames', all.x=TRUE, all.y=FALSE)
Gender_Names$Repec_F = Gender_Names$Repec_F*1

# URAP Audit
Gender_Names = merge(Gender_Names, Gender_URAP, by.x='Name', by.y='Name', all.x=TRUE, all.y=FALSE)

# Chinese names
Gender_Names = merge(Gender_Names, ChineseGivenNames, by.x='FirstName', by.y='Pinyin', all.x=TRUE, all.y=FALSE)
Gender_Names$CommonCH_Given[is.na(Gender_Names$CommonCH_Given)] <- 0

Gender_Names = merge(Gender_Names, ChineseSurnames, by.x='LastName', by.y='Pinyin', all.x=TRUE, all.y=FALSE)
Gender_Names$CommonCH_Surname[is.na(Gender_Names$CommonCH_Surname)] <- 0



##### Create flags #####
# The flags use arithmetic rather than logicals
# This makes the NaNs clearer

# Create consistency flags
Gender_Names$ConsistentF =  (Gender_Names$pFemale_German>=0.99) * 
                            (Gender_Names$pMale_SSA<=0.01)
Gender_Names$ConsistentF[Gender_Names$EEA_F==1 | Gender_Names$Repec_F==1] = 1
  
Gender_Names$ConsistentM =  (Gender_Names$pMale_German>=0.99) *
                            (Gender_Names$pMale_SSA>=0.99) *
                            (Gender_Names$Repec_F==0) *
                            (Gender_Names$EEA_F==0)


# Create flag for single confident gender: if either SSA or German is confident, and the other doesn't disagree
Gender_Names$SingleF =  (Gender_Names$pFemale_German>=0.99 | Gender_Names$pMale_SSA<=0.01) *
                        (Gender_Names$pMale_German<0.50 | is.na(Gender_Names$pMale_German)) *
                        (Gender_Names$pMale_SSA<0.50 | is.na(Gender_Names$pMale_SSA)) *
                        (Gender_Names$ConsistentF!=1 | is.na(Gender_Names$ConsistentF!=1))
                        

Gender_Names$SingleM =  (Gender_Names$pMale_German>=0.99 | Gender_Names$pMale_SSA>=0.99) *
                        (Gender_Names$pFemale_German<0.50 | is.na(Gender_Names$pFemale_German)) *
                        (Gender_Names$pMale_SSA>0.50 | is.na(Gender_Names$pMale_SSA)) *
                        (Gender_Names$ConsistentM!=1 | is.na(Gender_Names$ConsistentF)) *
                        (Gender_Names$Repec_F==0) *
                        (Gender_Names$EEA_F==0)

#assert_that(mean(Gender_Names$SingleF[which(Gender_Names$ConsistentF==1)]==0)==1)
assert_that(mean(Gender_Names$SingleM[which(Gender_Names$ConsistentM==1)]==0)==1)

# Create flag for no decisive gender coding
Gender_Names$Ambiguous <- (is.na(Gender_Names$SingleF) | Gender_Names$SingleF==0) *
                          (is.na(Gender_Names$SingleM) | Gender_Names$SingleM==0) *
                          (is.na(Gender_Names$ConsistentF) | Gender_Names$ConsistentF==0) *
                          (is.na(Gender_Names$ConsistentM) | Gender_Names$ConsistentM==0)

# Create final flags
Gender_Names$Gender_F <- (Gender_Names$ConsistentF==1 | Gender_Names$Gender_URAP=="F") * 1
Gender_Names$Gender_F[is.na(Gender_Names$Gender_F)] <- 0

Gender_Names$Gender_M <- (Gender_Names$ConsistentM==1 | Gender_Names$SingleM==1 | Gender_Names$Gender_URAP=="M") * 1
Gender_Names$Gender_M[is.na(Gender_Names$Gender_M)] <- 0
Gender_Names$Gender_M[Gender_Names$Gender_F==1] <- 0

Gender_Names$Gender_A <- 0
Gender_Names$Gender_A[Gender_Names$Gender_F + Gender_Names$Gender_M == 0] <- 1
Gender_Names$Gender_A[Gender_Names$Gender_F + Gender_Names$Gender_M == 2] <- 1

Gender_Names$Gender_F[Gender_Names$Gender_A==1] <- 0
Gender_Names$Gender_M[Gender_Names$Gender_A==1] <- 0


assert_that(mean(Gender_Names$Gender_F + Gender_Names$Gender_M + Gender_Names$Gender_A) == 1)
assert_that(max(Gender_Names$Gender_F + Gender_Names$Gender_M + Gender_Names$Gender_A) == 1)

# Drop duplicates
Gender_Names = Gender_Names[!duplicated(Gender_Names$Name),]

# Attach EconLit and EE flags
Gender_Names <- merge(Gender_Names, EconLitName, by.x='Name', by.y='Name', all.x=TRUE)
Gender_Names$EconLit[is.na(Gender_Names$EconLit)]<-0

Gender_Names <- merge(Gender_Names, EEAuth, by.x='Name', by.y='Name', all.x=TRUE)
Gender_Names$EEAuth[is.na(Gender_Names$EEAuth)]<-0

Gender_Names <- merge(Gender_Names, EERef, by.x='Name', by.y='Name', all.x=TRUE)
Gender_Names$EERef[is.na(Gender_Names$EERef)]<-0


# Create string category
Gender_Names$URAP<-!is.na(Gender_Names$Gender_URAP)
Gender_Names$Gender_Cat <- ''

for (urap in 0:1) {
  
  if (urap==1) {
    tagURAP = 'Hand'
  } else {
    tagURAP = 'Data'
  }

  Gender_Names$Gender_Cat[Gender_Names$URAP==urap & Gender_Names$Gender_M==1] <- paste0('M, ', tagURAP)
  Gender_Names$Gender_Cat[Gender_Names$URAP==urap & Gender_Names$Gender_F==1] <- paste0('F, ', tagURAP)
  
}
Gender_Names$Gender_Cat[Gender_Names$Gender_A==1] <- paste0('U')
Gender_Names$Gender_Cat[(Gender_Names$EEA_F==1 | Gender_Names$Repec_F==1)] <- paste('EEA/RePEc')
Gender_Names$Gender_Cat[(Gender_Names$CommonCH_Given==1)] <- paste('Common CH')

# Overrides
Gender_Names$Gender_Cat[Gender_Names$Name=='Adair Morse'] <- 'F, Hand'
Gender_Names$Gender_Cat[Gender_Names$Name=='Kyle Bagwell'] <- 'M, Hand'


# Reconstruct german probability
GermanSum<- Gender_Names$pMale_German+Gender_Names$pFemale_German<0.8
Gender_Names$pMale_German[GermanSum] <- NA
Gender_Names$pFemale_German[GermanSum] <- NA

Gender_Names$pMale_German2 <- 1-Gender_Names$pFemale_German
  
Gender_Names$pMale_German[Gender_Names$pMale_German<0.5 & !is.na(Gender_Names$pMale_German)] <- 
  Gender_Names$pMale_German2[Gender_Names$pMale_German<0.5 & !is.na(Gender_Names$pMale_German)]

# Create bins
breaks <- c(0, 0.01, 0.05, .1, .2, .3, .4, .5, .6, .7, .8, .9, .95, .99, 1.01)
labels <- c(0, 0.01, 0.05, .1, .2, .3, .4, .5, .6, .7, .8, .9, .95, .99)
Gender_Names$pMale_German_Bin <- cut(Gender_Names$pMale_German, breaks, right=FALSE, labels=labels)
Gender_Names$pMale_SSA_Bin <- cut(Gender_Names$pMale_SSA, breaks, right=FALSE, labels=labels)



##### Output #####

# Save
write_dta(Gender_Names, path='temp/Gender_Names_Full.dta', version=13)

Gender_Names = select(Gender_Names, c(Name, Gender_Cat, pMale_SSA_Bin, pMale_German_Bin, EconLit, 
                                      Gender_M, Gender_F, Gender_A))

write_dta(Gender_Names, path='temp/Gender_Names.dta', version=13)

proc.time() - startTime
